//
//  MegacoolFFI.h
//  Megacool
//
//  Created by Nicolaj Broby Petersen on 4/26/16.
//  Copyright © 2016 Nicolaj Broby Petersen. All rights reserved.
//

#import <stdbool.h>

// The layouts of these structs must not be changed without syncing with
// the definitions in the unity wrapper in Megacool.cs.
typedef struct MegacoolLinkClickedEvent {
    int isFirstSession;
    const char *userId;
    const char *shareId;
    const char *url;
} MegacoolLinkClickedEvent;

typedef struct MegacoolReceivedShareOpenedEvent {
    const char *userId;
    const char *shareId;
    int state;
    double createdAt;
    double updatedAt;
    const void *eventDataBytes;
    int eventDataLength;
    const void *shareDataBytes;
    int shareDataLength;
    const char *url;
    int isFirstSession;
} MegacoolReceivedShareOpenedEvent;

typedef struct MegacoolSentShareOpenedEvent {
    const char *userId;
    const char *shareId;
    int state;
    double createdAt;
    double updatedAt;
    const char *receiverUserId;
    const char *url;
    int isFirstSession;
    const void *eventDataBytes;
    int eventDataLength;
    const void *shareDataBytes;
    int shareDataLength;
} MegacoolSentShareOpenedEvent;

typedef struct MegacoolUnityShare {
    const char *userId;
    const char *shareId;
    int state;
    double createdAt;
    double updatedAt;
    const void *dataBytes;
    int dataLength;
} MegacoolUnityShare;

typedef struct mcl_share_config {
    const char *recording_id;
    const char *message;
    const char *fallback_image;
    const char *url;
    const char *jsonData;
    int strategy;
} mcl_share_config;

typedef struct mcl_recording_config {
    const char *recording_id;
    int overflow_strategy;
    int max_frames;
    int frame_rate;
    int playback_frame_rate;
    int last_frame_delay;
    double peak_location;
    const char *last_frame_overlay;
} mcl_recording_config;

typedef enum mcl_capture_method {
    MCL_CAPTURE_METHOD_VIEW = 0,
    MCL_CAPTURE_METHOD_OPENGLES2,
    MCL_CAPTURE_METHOD_OPENGLES3,
    MCL_CAPTURE_METHOD_METAL,
} mcl_capture_method;

typedef enum mcl_gif_color_table {
    MCL_GIF_COLOR_TABLE_DYNAMIC = 0,
    MCL_GIF_COLOR_TABLE_FIXED,
    MCL_GIF_COLOR_TABLE_ANALYZE_FIRST,
} mcl_gif_color_table;

typedef void (*MegacoolDidCompleteShareDelegate)(void);
typedef void (*MegacoolDidDismissShareViewDelegate)(void);
typedef void (*EventHandlerDelegate)(const void *, int);
typedef void (*OnLinkClickedEvent)(MegacoolLinkClickedEvent);
typedef void (*OnReceivedShareOpenedEvent)(MegacoolReceivedShareOpenedEvent);
typedef void (*OnSentShareOpenedEvent)(MegacoolSentShareOpenedEvent);
typedef void (*OnRetrievedShares)(MegacoolUnityShare[], int);

void mcl_start_with_app_config(const char *appConfig, const char* wrapper, const char* version);

void mcl_manual_application_did_become_active(void);


#pragma mark - Recording and capture
void mcl_start_recording(const mcl_recording_config *config);

void mcl_register_score_change(int scoreDelta);

void mcl_capture_frame(const mcl_recording_config *config, bool force_add);

void mcl_pause_recording(void);

void mcl_stop_recording(void);

void mcl_delete_recording(const char *recordingId);

int mcl_get_number_of_frames(const char *recordingId);


#pragma mark - Shares
void mcl_present_share(const mcl_share_config *config);

void mcl_present_share_to_messenger(const mcl_share_config *config);

void mcl_present_share_to_twitter(const mcl_share_config *config);

void mcl_present_share_to_messages(const mcl_share_config *config);

void mcl_present_share_to_mail(const mcl_share_config *config);

void mcl_delete_shares(bool (*filter)(MegacoolUnityShare share));


#pragma mark - Configuration

void mcl_set_default_share_config(mcl_share_config *config);

void mcl_set_default_recording_config(mcl_recording_config *config);

void mcl_set_capture_method(mcl_capture_method captureMethod);

void mcl_set_capture_method_with_scale_factor(mcl_capture_method captureMethod, double scaleFactor);

void mcl_set_keep_completed_recordings(bool keep);

void mcl_set_gif_color_table(mcl_gif_color_table gifColorTable);


#pragma mark - Frame Enumeration API
typedef struct mcl_preview_data mcl_preview_data;

typedef struct mcl_preview_frame {
    char *path;
    int delay_ms;
} mcl_preview_frame;

mcl_preview_data *mcl_get_preview_for_recording(const char *recordingIdCString);

mcl_preview_frame *mcl_get_next_frame(mcl_preview_data *preview);

int mcl_get_preview_frame_count(mcl_preview_data *preview);

void mcl_free_preview_data(mcl_preview_data *preview);

void mcl_free_preview_frame(mcl_preview_frame* frame);


#pragma mark - Debugging/development
void mcl_submit_debug_data_with_message(const char *message);

void mcl_reset_identity(void);

void mcl_set_debug_mode(bool debugMode);

bool mcl_get_debug_mode(void);

